(define-module (ui net-l2tp ajax)
               :use-module (srfi srfi-1)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator algo)
               :export (on-load))

; DEBUG STRING (PASTE IN FOR YOUR NEEDS).
; (format #t "XXX: ~S\n" rdp_profile_name)
; (format #t "[ajax, module] action info\n")

; TO DEBUG DO:
; 1) define *debug* to #t
; 2) start ahttpd -l -d
(define *debug* #f)
(define (dmsg msg . args)
  (if *debug*
    (format #t "[ajax, net-l2tp]: ~S ~S\n" msg args)))



; READ CURRENT SETTINGS
(define (read-all)

  (dmsg "read-all() -- enter\n")

  ; Hide config area
  (form-update-visibility "area_config" #f)

  (and

    ; FILL LISTS
    (for-each
      (lambda(lst)
        (dmsg "process list\n" lst)

        (let* ((name (car lst))
               (select_first (car (cdr lst)))
               (data (woo-list (string-append "/net-l2tp/" name)
                               'name name
                               'language (form-value "language"))))
          (form-update-enum name data)
          ; take first, make it active
          (and
            select_first
            (pair? data)
            ; select first item only for general enums, not multi-select lists
            (= (length (car data)) 5)
            (let ((xxx (woo-get-option (car data) 'name)))
              (form-update-value name (if (string? xxx) xxx ""))))
          ))
      ; put here all lists in form ("list_name" bool-should-select-firts-element)
      '(("list_interfaces" #t) ("list_possible_hosts" #t)))

    ; REBIND ACTIONS ON LISTS
    (form-bind "list_interfaces" "change" read-current-iface)))


; RESET ALL UI WIDGETS TO EMPTY
(define (widgets-reset)

  (dmsg "widgets-reset() -- enter\n")

  (for-each (lambda(p) (form-update-value p ""))
            '("input_server"
              "input_username"
              "input_password"
              "input_new_iface_name"))

  ; Reset to default reasonable values
  (form-update-value "bool_default_route" #t)
  (form-update-value "bool_auto_start" #t)

  ; Visibility
  (form-update-visibility "btn_up" #f)
  (form-update-visibility "btn_down" #f)

  ; Reset labels
  (for-each (lambda(p) (form-update-value p (_ "N/A")))
            '("label_status"
              "label_status_remote_ip"
              "label_status_local_ip"))

  ; Lists
  (form-update-value "list_possible_hosts" "do_not_use"))


; READ INTERFACE SETTINGS
(define (read-iface iface)

  (dmsg "read-iface() -- enter\n")

  ; RESET WIDGETS
  (widgets-reset)

  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/net-l2tp/status" 'iface iface 'language (form-value "language")))
             (state (woo-get-option data 'connection_state)))
        (form-update-value-list '("label_status"
                                  "label_status_local_ip"
                                  "label_status_remote_ip"
                                  "input_server"
                                  "input_username"
                                  "input_password"
                                  "list_possible_hosts"
                                  "bool_default_route"
                                  "bool_auto_start")
                                data)
        (and
          (string? state)
          (not (string-null? state))
          (if (equal? state "is_up") (form-update-visibility "btn_down" #t))
          (if (equal? state "is_down") (form-update-visibility "btn_up" #t)))))))


; READ CURRENT INTERFACE/CONNECTION SETTINGS
(define (read-current-iface)

  (dmsg "read-current-iface() -- enter\n")

  (let* ((iface (form-value "list_interfaces")))
    (and
      (string? iface)
      (read-iface iface)
      (form-update-visibility "area_config" #t))))


; SAVE L2TP CONNECTION SETTINGS
(define (iface-save-settings)

  (dmsg "iface-save-settings() -- enter\n")

  (catch/message
    (lambda()
      (apply woo-write "/net-l2tp/connection_save"
             'bool_auto_start (form-value "bool_auto_start")
             'bool_default_route (form-value "bool_default_route")
             (form-value-list))
      (read-iface (form-value "list_interfaces")))))


; REMOVE L2TP CONNECTION
(define (iface-remove)

  (dmsg "iface-remove() -- enter\n")

  (catch/message
    (lambda()
      (woo-write "/net-l2tp/connection_remove" 'iface (form-value "list_interfaces"))
      (read-all)
      (read-current-iface))))


; ADD NEW L2TP CONNECTION
(define (iface-add-new)

  (dmsg "iface-add-new() -- enter\n")

  (let ((name (form-value "input_new_iface_name")))
    (and
      (string? name)
      (not (string-null? name))
      (catch/message
        (lambda()
          (woo-write "/net-l2tp/connection_new" 'input_new_iface_name name)))
      (read-all)
      (form-update-value "list_interfaces" name)
      (read-current-iface))))


; BRING INTERFACE UP
(define (iface-up)

  (dmsg "iface-up() -- enter\n")

  (catch/message
    (lambda()
      (woo-call "/net-l2tp/connection_up" 'iface (form-value "list_interfaces"))))
    (read-current-iface))


; BRING INTERFACE DOWN
(define (iface-down)

  (dmsg "iface-down () -- enter\n")

  (catch/message
    (lambda()
      (woo-call "/net-l2tp/connection_down" 'iface (form-value "list_interfaces"))))
    (read-current-iface))


; ENTRY POINT
(define (on-load)
  (form-bind "btn_create" "click" iface-add-new)
  (form-bind "btn_delete" "click" iface-remove)
  (form-bind "btn_save" "click" iface-save-settings)
  (form-bind "btn_reset" "click" read-current-iface)
  (form-bind "btn_up" "click" iface-up)
  (form-bind "btn_down" "click" iface-down)
  (form-bind "list_interfaces" "change" read-current-iface)
  (read-all)
  (read-current-iface))

; vim: ft=lisp expandtab
